==> appy-1.0.2.tar.gz: appy-1.0.2/py3/appy/pod/xhtml/parser.py <==
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Copyright (C) 2007-2020 Gaetan Delannay

# This file is part of Appy.

# Appy is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.

# Appy is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# Appy. If not, see <http://www.gnu.org/licenses/>.

from appy.xml.escape import Escape
from appy.xml import Environment, Parser
from appy.pod.xhtml import tags, visitors

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CHUNK_NOT_WITH_FILE = 'Cannot parse a file when chunk=True, only a string.'

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
class XhtmlEnvironment(Environment):
    def __init__(self):
        # Call the base constructor
        Environment.__init__(self)
        # The parsing result will store a root Tag instance
        self.r = None
        # The currently walked tag (a tags.Tag or sub-class' instance)
        self.current = None
        # The current content (a tags.Content instance)
        self.content = None
        # Tags encountered so far, by type
        self.tags = {}
        # The main parser (see below)
        self.parser = None

    def addTag(self, tag):
        '''A new p_tag has been encountered. Add it in the environment's local
           data structures.'''
        # Store it as the current tag
        if not self.r:
            # It is the root tag
            self.r = tag
            isRoot = True
        else:
            isRoot = False
        self.current = tag
        # Store it in self.tags if walkable. Do not store the root tag: it is a
        # tag that was artificially added to ensure there is a single root tag.
        class_ = tag.__class__
        if not isRoot and class_.walkable:
            name = class_.__name__
            if name in self.tags:
                self.tags[name].append(tag)
            else:
                self.tags[name] = [tag]

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
class XhtmlParser(Parser):
    '''Creates a tree of Tag elements from a chunk of XHTML code as a string'''

    def __init__(self, env=None, caller=None, chunk=False, compress=True,
                 normalizeTables=True, keepWithNext=0,
                 removeTrailingParas=None):
        # Define a default environment if p_env is None
        env = env or XhtmlEnvironment()
        # Call the base constructor
        Parser.__init__(self, env, caller)
        # If the XHTML to parse is not a complete, single-root-tag XML, but,
        # instead, a chunk of XHTML potentially containing several tags at the
        # root level, p_chunk is True and the XHTML chunk will be surrounded by
        # a faxe "x" tag.
        self.chunk = chunk
        # By default, "compress" is True: the parser will remove ignorable
        # whitespace to produce a shorter output. If it is a requirement to keep
        # the general formatting of the input XHTML, set p_compress to False.
        self.compress = compress
        # Define the visitors that will walk the tree of tags we will build
        activated = []
        if normalizeTables:
            activated.append(visitors.TablesNormalizer())
        if keepWithNext:
            activated.append(visitors.KeepWithNext(keepWithNext))
        if removeTrailingParas is not None:
            cleaner = visitors.Cleaner(removeTrailingParas=removeTrailingParas)
            activated.append(cleaner)
        self.visitors = activated
        # This flag is True if the input XHTML has been modified by at least one
        # visitor.
        self.updated = False

    def parse(self, xml, source='string'):
        '''Surrounds p_xml with a fake root tag if p_self.chunk is True, before
           calling the base method.'''
        # Surrounding with a fake tag is only possible if p_source is "string"
        chunk = self.chunk
        if chunk:
            if source == 'string':
                xml = '<x>%s</x>' % xml
            else:
                raise Exception(CHUNK_NOT_WITH_FILE)
        # Call the base method
        return Parser.parse(self, xml, source=source)

    def startElement(self, elem, attrs):
        '''Manages the start of a tag'''
        env = self.env
        # Clean current content
        env.content = None
        # Create the corresponding Tag instance
        tag = tags.get(elem)(elem, attrs, parent=env.current)
        env.addTag(tag)

    def endElement(self, elem):
        '''Manages the end of a tag'''
        env = self.env
        # Clean current content
        env.content = None
        # We are back to p_elem's parent
        env.current = env.current.parent

    def characters(self, text):
        '''Manages text encountered within the current tag'''
        env = self.env
        # Ignore p_text if directly contained in a tag that is not supposed to
        # directly contain content. In that case, p_text is probably ignorable
        # whitespace. Avoid doing this if p_self.compress is False.
        if env.current.structural and self.compress: return
        # Xhtml-escape p_text and store it on the environment
        text = Escape.xml(text)
        if env.content:
            env.content.text += text
        else:
            # Create a Content instance to store p_text
            content = env.content = tags.Content(text)
            # Add content as child of the current tag
            tag = env.current
            if tag.children:
                tag.children.append(content)
            else:
                tag.children = [content]

    def endDocument(self):
        '''Converts p_self.env.r into a chunk of ODF code and store it in
           p_self.res.'''
        env = self.env
        # Accept the visitors
        for visitor in self.visitors:
            updated = visitor.visit(env)
            self.updated = self.updated or updated
        self.r = env.r.asXhtml()
        if self.chunk:
            # Remove the base tag
            self.r = self.r[3:-4]
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

==> appy-1.0.3.tar.gz: appy-1.0.3/py3/appy/pod/xhtml/parser.py <==
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Copyright (C) 2007-2020 Gaetan Delannay

# This file is part of Appy.

# Appy is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.

# Appy is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# Appy. If not, see <http://www.gnu.org/licenses/>.

from appy.xml.escape import Escape
from appy.xml import Environment, Parser
from appy.pod.xhtml import tags, visitors

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CHUNK_NOT_WITH_FILE = 'Cannot parse a file when chunk=True, only a string.'

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
class XhtmlEnvironment(Environment):
    def __init__(self):
        # Call the base constructor
        Environment.__init__(self)
        # The parsing result will store a root Tag instance
        self.r = None
        # The currently walked tag (a tags.Tag or sub-class' instance)
        self.current = None
        # The current content (a tags.Content instance)
        self.content = None
        # Tags encountered so far, by type
        self.tags = {}
        # The main parser (see below)
        self.parser = None

    def addTag(self, tag):
        '''A new p_tag has been encountered. Add it in the environment's local
           data structures.'''
        # Store it as the current tag
        if not self.r:
            # It is the root tag
            self.r = tag
            isRoot = True
        else:
            isRoot = False
        self.current = tag
        # Store it in self.tags if walkable. Do not store the root tag: it is a
        # tag that was artificially added to ensure there is a single root tag.
        class_ = tag.__class__
        if not isRoot and class_.walkable:
            name = class_.__name__
            if name in self.tags:
                self.tags[name].append(tag)
            else:
                self.tags[name] = [tag]

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
class XhtmlParser(Parser):
    '''Creates a tree of Tag elements from a chunk of XHTML code as a string'''

    def __init__(self, env=None, caller=None, chunk=False, encoded=False,
                 compress=True, normalizeTables=True, optimizeTables=False,
                 keepWithNext=0, removeTrailingParas=None):
        # Define a default environment if p_env is None
        env = env or XhtmlEnvironment()
        # Call the base constructor
        Parser.__init__(self, env, caller)
        # If the XHTML to parse is not a complete, single-root-tag XML, but,
        # instead, a chunk of XHTML potentially containing several tags at the
        # root level, p_chunk is True and the XHTML chunk will be surrounded by
        # a faxe "x" tag.
        self.chunk = chunk
        # By default, "compress" is True: the parser will remove ignorable
        # whitespace to produce a shorter output. If it is a requirement to keep
        # the general formatting of the input XHTML, set p_compress to False.
        self.compress = compress
        # Define the visitors that will walk the tree of tags we will build
        activated = []
        if normalizeTables:
            activated.append(visitors.TablesNormalizer())
        if optimizeTables:
            activated.append(visitors.TablesOptimizer())
        if keepWithNext:
            activated.append(visitors.KeepWithNext(keepWithNext))
        if removeTrailingParas is not None:
            cleaner = visitors.Cleaner(removeTrailingParas=removeTrailingParas)
            activated.append(cleaner)
        self.visitors = activated
        # This flag is True if the input XHTML has been modified by at least one
        # visitor.
        self.updated = False

    def parse(self, xml, source='string'):
        '''Surrounds p_xml with a fake root tag if p_self.chunk is True, before
           calling the base method.'''
        # Surrounding with a fake tag is only possible if p_source is "string"
        chunk = self.chunk
        if chunk:
            if source == 'string':
                xml = '<x>%s</x>' % xml
            else:
                raise Exception(CHUNK_NOT_WITH_FILE)
        # Call the base method
        return Parser.parse(self, xml, source=source)

    def startElement(self, elem, attrs):
        '''Manages the start of a tag'''
        env = self.env
        # Clean current content
        env.content = None
        # Create the corresponding Tag instance
        tag = tags.get(elem)(elem, attrs, parent=env.current)
        env.addTag(tag)

    def endElement(self, elem):
        '''Manages the end of a tag'''
        env = self.env
        # Clean current content
        env.content = None
        # We are back to p_elem's parent
        env.current = env.current.parent

    def characters(self, text):
        '''Manages text encountered within the current tag'''
        env = self.env
        # Ignore p_text if directly contained in a tag that is not supposed to
        # directly contain content. In that case, p_text is probably ignorable
        # whitespace. Avoid doing this if p_self.compress is False.
        if env.current.structural and self.compress: return
        # Xhtml-escape p_text and store it on the environment
        text = Escape.xml(text)
        if env.content:
            env.content.text += text
        else:
            # Create a Content instance to store p_text
            content = env.content = tags.Content(text)
            # Add content as child of the current tag
            tag = env.current
            if tag.children:
                tag.children.append(content)
            else:
                tag.children = [content]

    def endDocument(self):
        '''Converts p_self.env.r into a chunk of ODF code and store it in
           p_self.res.'''
        env = self.env
        # Accept the visitors
        for visitor in self.visitors:
            updated = visitor.visit(env)
            self.updated = self.updated or updated
        self.r = env.r.asXhtml()
        if self.chunk:
            # Remove the base tag
            self.r = self.r[3:-4]
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
